@@ -1,7 +1,7 @@
 
 /*
  *  expState.c
- *  $Id: expState.c,v 1.28 2001/03/27 05:00:08 bkorb Exp $
+ *  $Id: expState.c,v 1.29 2001/04/30 03:38:32 bkorb Exp $
  *  This module implements expression functions that
  *  query and get state information from AutoGen data.
  */
@@ -93,7 +93,7 @@ find_entry_value( SCM op, SCM obj, SCM test )
     tSCC        zFailed[] = "failed\n";
     tSCC        zSucc[]   = "SUCCESS\n";
     char*       pzField;
-    tDefEntry*  pSavDef = pDefContext;
+    SCM         result    = SCM_BOOL_F;
 
     char* pzName = gh_scm2newstr( obj, NULL );
 
@@ -112,7 +112,7 @@ find_entry_value( SCM op, SCM obj, SCM test )
     if (pE == (tDefEntry*)NULL) {
         if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS)
             fputs( zFailed, pfTrace );
-        goto return_FALSE;
+        goto return_res;
     }
 
     /*
@@ -122,35 +122,25 @@ find_entry_value( SCM op, SCM obj, SCM test )
         if (pE->valType != VALTYP_TEXT) {
             if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS)
                 fputs( zFailed, pfTrace );
-            goto return_FALSE;
+            goto return_res; /* Cannot match string -- not a text value */
         }
 
-        field = gh_str02scm( pE->pzValue );
-        if (isIndexed) {
-            field = gh_call2( op, field, test );
-            if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS)
-                fputs( (field == SCM_BOOL_T) ? zSucc : zFailed, pfTrace );
-            free( (void*)pzName );;
-            return field;
-        }
+        field  = gh_str02scm( pE->pzValue );
+        result = gh_call2( op, field, test );
+        if (! isIndexed)
+            while (result == SCM_BOOL_F) {
 
-        for (;;) {
-            if (gh_call2( op, field, test ) == SCM_BOOL_T) {
-                if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS)
-                    fputs( zSucc, pfTrace );
-                goto return_TRUE;
+                pE = pE->pTwin;
+                if (pE == (tDefEntry*)NULL)
+                    break;
+
+                field = gh_str02scm( pE->pzValue );
+                result = gh_call2( op, field, test );
             }
 
-            pE = pE->pTwin;
-            if (pE == (tDefEntry*)NULL)
-                break;
-
-            field = gh_str02scm( pE->pzValue );
-        }
-
         if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS)
-            fputs( zFailed, pfTrace );
-        goto return_FALSE;
+            fputs( (result == SCM_BOOL_T) ? zSucc : zFailed, pfTrace );
+        goto return_res;
     }
 
     /*
@@ -159,7 +149,7 @@ find_entry_value( SCM op, SCM obj, SCM test )
     if (pE->valType == VALTYP_TEXT) {
         if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS)
             fputs( zFailed, pfTrace );
-        goto return_FALSE;
+        goto return_res;
     }
 
     /*
@@ -167,34 +157,30 @@ find_entry_value( SCM op, SCM obj, SCM test )
      */
     pzField[-1] = '.';
     field       = gh_str02scm( pzField );
-    pDefContext = (tDefEntry*)(void*)(pE->pzValue);
+    {
+        tDefStack    stack   = currDefCtx;
+        currDefCtx.pPrev     = &stack;
 
-    if (isIndexed) {
-	if (find_entry_value( op, field, test ) == SCM_BOOL_T)
-	    goto return_TRUE;
-	goto return_FALSE;
+        currDefCtx.pDefs = (tDefEntry*)(void*)(pE->pzValue);
+        result = find_entry_value( op, field, test );
+
+        if (! isIndexed)
+            while (result == SCM_BOOL_F) {
+
+                pE = pE->pTwin;
+                if (pE == (tDefEntry*)NULL)
+                    break;
+
+                currDefCtx.pDefs = (tDefEntry*)(void*)(pE->pzValue);
+                result = find_entry_value( op, field, test );
+            }
+
+        currDefCtx = stack;
     }
 
-    for (;;) {
-        if (find_entry_value( op, field, test ) == SCM_BOOL_T)
-            goto return_TRUE;
-
-        pE = pE->pTwin;
-        if (pE == (tDefEntry*)NULL)
-            break;
-
-        pDefContext = (tDefEntry*)(void*)(pE->pzValue);
-    }
-
- return_FALSE:
-    pDefContext = pSavDef;
+ return_res:
     free( (void*)pzName );
-    return SCM_BOOL_F;
-
- return_TRUE:
-    pDefContext = pSavDef;
-    free( (void*)pzName );
-    return SCM_BOOL_T;
+    return result;
 }
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
