@@ -1,6 +1,6 @@
 
 /*
- *  $Id: funcDef.c,v 1.34 2001/04/15 20:13:53 bkorb Exp $
+ *  $Id: funcDef.c,v 1.35 2001/04/30 03:38:32 bkorb Exp $
  *
  *  This module implements the DEFINE text function.
  */
@@ -39,7 +39,7 @@ struct def_list {
     char*      pzExpr;
 };
 
-tSCC  zNoResolution[] = "Could not resolve INVOKE macro name - %s";
+tSCC  zNoResolution[] = "Could not resolve macro name: ``%s''";
 tSCC zTplInvoked[] = "Template macro %s invoked with %d args\n";
 #endif /* not defined DEFINE_LOAD_FUNCTIONS */
 
@@ -598,8 +598,8 @@ MAKE_HANDLER_PROC( Debug )
 
     if (OPT_VALUE_TRACE > TRACE_NOTHING)
         fprintf( pfTrace, "  --  DEBUG %s -- FOR index %d\n",
-                 pT->pzTemplText + pMac->ozText,
-                 (forInfo.for_depth <= 0) ? -1 : forInfo.for_index );
+                 pT->pzTemplText + pMac->ozText, (forInfo.fi_depth <= 0) ? -1
+                 : forInfo.fi_data[ forInfo.fi_depth - 1].for_index );
     /*
      *  The case element values were chosen to thwart most
      *  optimizer that might be too bright for its own good.
@@ -683,6 +683,90 @@ anonymousTemplate( void* p )
 }
 
 
+/*
+ *  build_defs
+ *
+ *  Build up a definition context created by passed-in macro arguments
+ */
+STATIC void
+build_defs( int defCt, tDefList* pList )
+{
+    tDefEntry* pDefs = &(pList->de);
+	currDefCtx.pDefs = pDefs;
+
+    /*
+     *  FOR each definition, evaluate the associated expression
+     *      and set the text value to it.
+     */
+    do  {
+        if (pList->pzExpr == (char*)NULL)
+            continue;
+
+    retryExpression:
+        switch (*(pList->pzExpr)) {
+        case ';':
+        {
+            char* pz = strchr( pList->pzExpr, '\n' );
+            if (pz != (char*)NULL) {
+                while (isspace( *++pz ))  ;
+                pList->pzExpr = pz;
+                goto retryExpression;
+            }
+            /* FALLTHROUGH */
+        }
+        case NUL:
+            pList->pzExpr = (char*)NULL;
+            pList->de.pzValue = "";
+            break;
+
+        case '[':
+            if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS)
+                fprintf( pfTrace, "anonymus template arg %d\n",
+                         pCurMacro->sibIndex - defCt );
+
+            pList->de.pzValue =
+                anonymousTemplate(
+                    (void*)strtoul( pList->pzExpr + 2, (char**)NULL, 0 ));
+            break;
+
+        case '(':
+        {
+            SCM res;
+
+            /*
+             *  It is a scheme expression.  Accept only string
+             *  and number results.
+             */
+            if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS) {
+                fprintf( pfTrace, "Scheme eval for arg %d:\n\t`%s'\n",
+                         pCurMacro->sibIndex - defCt, pList->pzExpr );
+            }
+            res = gh_eval_str( pList->pzExpr );
+
+            if (gh_string_p( res )) {
+                pList->de.pzValue = strdup( SCM_CHARS( res ));
+            }
+            else if (gh_number_p( res )) {
+                pList->de.pzValue = (char*)AGALOC( 16, "number buf" );
+                snprintf( pList->de.pzValue, 16, "%ld", gh_scm2long( res ));
+            }
+            else
+                pList->de.pzValue = strdup( "" );
+            break;
+        }
+
+        case '`':
+            if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS) {
+                fprintf( pfTrace, "shell eval for arg %d:\n\t`%s'\n",
+                         pCurMacro->sibIndex - defCt, pList->pzExpr+1 );
+            }
+            pList->de.pzValue = runShell( pList->pzExpr+1 );
+            break;
+        }
+    } while (pList++, --defCt > 0);
+}
+
+
 /*=macfunc DEFINE
  *
  *  what:    Define a user AutoGen macro
@@ -747,6 +831,8 @@ MAKE_HANDLER_PROC( Define )
     int         defCt  = pMac->sibIndex;
     SCM         res;
     tDefEntry*  pDefs;
+    tDefStack*  pStack;
+    tDefStack   stack;
 
     pT = (tTemplate*)pMac->funcPrivate;
 
@@ -760,91 +846,23 @@ MAKE_HANDLER_PROC( Define )
     /*
      *  IF we have no special definitions, then do not nest definitions
      */
-    if (defCt == 0)
-        pDefs = pDefContext;
-    else {
-        pDefs = &(pList->de);
-        pList->de.pDad = pDefContext;
-
-        /*
-         *  FOR each definition, evaluate the associated expression
-         *      and set the text value to it.
-         */
-        do  {
-            if (pList->pzExpr == (char*)NULL)
-                continue;
-
-        retryExpression:
-            switch (*(pList->pzExpr)) {
-            case ';':
-            {
-                char* pz = strchr( pList->pzExpr, '\n' );
-                if (pz != (char*)NULL) {
-                    while (isspace( *++pz ))  ;
-                    pList->pzExpr = pz;
-                    goto retryExpression;
-                }
-                /* FALLTHROUGH */
-            }
-            case NUL:
-                pList->pzExpr = (char*)NULL;
-                pList->de.pzValue = "";
-                break;
-
-            case '[':
-                if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS)
-                    fprintf( pfTrace, "anonymus template arg %d\n",
-                             pMac->sibIndex - defCt );
-
-                pList->de.pzValue =
-                    anonymousTemplate(
-                        (void*)strtoul( pList->pzExpr + 2, (char**)NULL, 0 ));
-                break;
-
-            case '(':
-                /*
-                 *  It is a scheme expression.  Accept only string
-                 *  and number results.
-                 */
-                if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS) {
-                    fprintf( pfTrace, "Scheme eval for arg %d:\n\t`%s'\n",
-                             pMac->sibIndex - defCt, pList->pzExpr );
-                }
-                res = gh_eval_str( pList->pzExpr );
-
-                if (gh_string_p( res )) {
-                    pList->de.pzValue = strdup( SCM_CHARS( res ));
-                }
-                else if (gh_number_p( res )) {
-                    pList->de.pzValue = (char*)AGALOC( 16, "number buf" );
-                    snprintf( pList->de.pzValue, 16, "%ld", gh_scm2long( res ));
-                }
-                else
-                    pList->de.pzValue = strdup( "" );
-                break;
-
-            case '`':
-                if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS) {
-                    fprintf( pfTrace, "shell eval for arg %d:\n\t`%s'\n",
-                             pMac->sibIndex - defCt, pList->pzExpr+1 );
-                }
-                pList->de.pzValue = runShell( pList->pzExpr+1 );
-                break;
-            }
-        } while (pList++, --defCt > 0);
+    if (defCt != 0) {
+        stack  = currDefCtx;
+        currDefCtx.pPrev     = &stack;
+        build_defs( defCt, pList );
     }
 
     {
-	tDefEntry*  pSavDef = pDefContext;
-	tTemplate*  pOldTpl = pCurTemplate;
-	pDefContext  = pDefs;
-	pCurTemplate = pT;
-    
-	generateBlock( pT, pT->aMacros, pT->aMacros + pT->macroCt );
-	pCurTemplate = pOldTpl;
-	pDefContext  = pSavDef;
+        tTemplate*  pOldTpl = pCurTemplate;
+        pCurTemplate = pT;
+
+        generateBlock( pT, pT->aMacros, pT->aMacros + pT->macroCt );
+        pCurTemplate = pOldTpl;
     }
 
+    if (defCt != 0)
+        currDefCtx = stack;
+
     if ((defCt = pMac->sibIndex) > 0) {
         pList = (tDefList*)pMac->res;
         while (defCt-- > 0) {
@@ -938,113 +956,9 @@ MAKE_HANDLER_PROC( Invoke )
 
     return mFunc_Define( pT, pMac );
 }
-
-/*=gfunc ag_invoke
- *
- * what:   invoke AutoGen macro
- * exparg: macro, name of macro to invoke
- * exparg: args,  macro arguments, optional, ellipsis
- *
- * doc:  Invoke an AutoGen macro and put the results into a string.
- *       As of this writing, however, the macro arguments are not
- *       well implemented.  Each entry of the list must be a string
- *       that starts with a name, followed by an equal sign followed
- *       by the text value.  This is not very Schemey.  Don't rely on this.
-=*/
-    SCM
-ag_scm_ag_invoke( SCM macName, SCM list )
-{
-    SCM         res = SCM_UNDEFINED, car;
-    tTemplate*  pT;
-    tMacro*     pMac;
-    char*       pz;
-
-    int  len   = scm_ilength( list );
-    int  defCt = 0;
-
-    tDefEntry*  pDefList;
-    tDefEntry*  pDE;
-
-    if (! gh_string_p( macName ))
-        LOAD_ABORT( pT, pMac, zNoResolution );
-
-    pT = findTemplate( SCM_CHARS( macName ));
-    if (pT == (tTemplate*)NULL)
-        LOAD_ABORT( pCurTemplate, pCurMacro, zNoResolution );
-
-    if (len <= 0) {
-        pDE = pDefContext;
-        pDefList = (tDefEntry*)NULL;
-    }
-
-    else {
-        pDefList = \
-        pDE = (tDefEntry*)AGALOC( sizeof( *pDE ) * len,
-                                  "named args to AG macro" );
-        memset( (void*)pDE, 0, sizeof( *pDE ) * len );
-        for (;;) {
-            if (len-- <= 0)
-                break;
-
-            defCt++;
-            car  = SCM_CAR( list );
-            list = SCM_CDR( list );
-            if (! gh_string_p( car ))
-                LOAD_ABORT( pCurTemplate, pCurMacro,
-                            "need name for macro arg" );
-            pz = pDE->pzDefName = strdup( SCM_CHARS( car ));
-            while (ISNAMECHAR( *pz ))  pz++;
-            while (isspace( *pz ))     pz++;
-            if (*pz != NUL) {
-                pDE++;
-                continue;
-            }
-
-            if (*pz != '=')
-                LOAD_ABORT( pCurTemplate, pCurMacro,
-                            "garbage in macro arg name" );
-
-            *(pz++) = NUL;
-            if (*pz != NUL) {
-                while (isspace( *pz ))  pz++;
-                pDE->pzValue = pz;
-                continue;
-            }
-
-            if (len-- <= 0)
-                LOAD_ABORT( pCurTemplate, pCurMacro,
-                            "missing value for macro arg" );
-
-            car  = SCM_CAR( list );
-            list = SCM_CDR( list );
-            if (! gh_string_p( car ))
-                LOAD_ABORT( pCurTemplate, pCurMacro,
-                            "need name for macro arg" );
-            pDE->pzValue = strdup( SCM_CHARS( car ));
-        }
-
-        if (defCt > 1)
-            /*
-             *  There was more than one value assignment.
-             *  Sort them just so we know the siblings are together.
-             *  Order is preserved by comparing string addresses,
-             *  if the strings compare equal.
-             */
-            qsort( (void*)pMac->res, defCt, sizeof( tDefList ), orderDefList );
-    }
-
-    /*
-     *  FIXME:  insert code that will
-     *  1.  Process any invocation arguments
-     *  2.  Divert to a _UNIQUE_ file name
-     *  3.  read back the diversion and set the result SCM to it
-     *  4.  delete the diversion file
-     *  5.  deallocate anything allocated
-     */
-    return res;
-}
 #endif /* DEFINE_LOAD_FUNCTIONS not defined */
 
+
 /* Load Debug
  *
  *  what:   Loads the debug function so you can set breakpoints
