@@ -1,7 +1,7 @@
 
 /*
  *  agTempl.c
- *  $Id: tpProcess.c,v 1.10 2001/03/27 05:00:07 bkorb Exp $
+ *  $Id: tpProcess.c,v 1.11 2001/04/30 03:38:32 bkorb Exp $
  *  Parse and process the template data descriptions
  */
 
@@ -89,7 +89,7 @@ processTemplate( tTemplate* pTF )
 {
     tFpStack fpRoot = { (tFpStack*)NULL, (FILE*)NULL, (char*)NULL };
 
-    forInfo.for_depth = 0;
+    forInfo.fi_depth = 0;
 
     /*
      *  IF the template file does not specify any output suffixes,
@@ -106,7 +106,6 @@ processTemplate( tTemplate* pTF )
             pCurFp        = &fpRoot;
             fpRoot.pFile  = stdout;
             AGDUPSTR( fpRoot.pzOutName, "stdout", "processTemplate" );
-	    pDefContext = (tDefEntry*)rootEntry.pzValue;
             generateBlock( pTF, pTF->aMacros, pTF->aMacros + pTF->macroCt );
             break;
         }
@@ -139,8 +138,7 @@ processTemplate( tTemplate* pTF )
             openOutFile( pOS, &fpRoot );
 
             pzCurSfx = pOS->zSuffix;
-	    pDefContext = (tDefEntry*)rootEntry.pzValue;
-
+            currDefCtx = rootDefCtx;
             generateBlock( pTF, pTF->aMacros, pTF->aMacros + pTF->macroCt );
 
             do  {
