@@ -1,6 +1,6 @@
 
 /*
- *  autoopts.h  $Id: autoopts.h,v 2.10 2000/03/15 16:01:57 bkorb Exp $
+ *  autoopts.h  $Id: autoopts.h,v 2.18 2000/10/29 01:43:31 bkorb Exp $
  *
  *  This file defines all the global structures and special values
  *  used in the automated option processing library.
@@ -59,12 +59,12 @@
  *  Bump this number *ONLY* if in incompatible change is made to
  *  the interface.
  */
-#define MIN_OPTION_VERSION  ((3  << 12) + 2)
+#define MIN_OPTION_VERSION  ((AO_CURRENT - AO_AGE) * 4096)
 
 /*
  *  Convert the number to a list usable in a printf call
  */
-#define NUM_TO_VER(n)       ((n) >> 12), ((n) & 0x0FFF)
+#define NUM_TO_VER(n)       ((n) >> 12), ((n) >> 7) & 0x001F, (n) & 0x007F
 
 #ifdef HAVE_SYS_TYPES_H
 #  include <sys/types.h>
@@ -100,7 +100,7 @@
 
 #ifndef FOPEN_BINARY_FLAG
 #  ifdef USE_FOPEN_BINARY
-#    define FOPEN_BINARY_FLAG	"b"
+#    define FOPEN_BINARY_FLAG   "b"
 #  else
 #    define FOPEN_BINARY_FLAG
 #  endif
@@ -108,7 +108,7 @@
 
 #ifndef FOPEN_TEXT_FLAG
 #  ifdef USE_TEXT_BINARY
-#    define FOPEN_TEXT_FLAG	"t"
+#    define FOPEN_TEXT_FLAG     "t"
 #  else
 #    define FOPEN_TEXT_FLAG
 #  endif
@@ -122,27 +122,6 @@
 #  define STATIC static
 #endif
 
-#ifndef MSDOS
-#  define DIR_SEP_CHAR   '/'
-#  define OPT_CHAR       '-'
-#else
-#  define DIR_SEP_CHAR   '\\'
-#  define OPT_CHAR       '/'
-#endif
-
-#ifndef HAVE_PATHFIND
-/*
- * pathfind looks for a a file with name FILENAME and MODE access
- * along colon delimited PATH, and returns the full pathname as a
- * string, or NULL if not found.
- */
-#  ifdef __STDC__
-     extern char* pathfind(const char *, const char *, const char *);
-#  else
-     extern char* pathfind();
-#  endif
-#endif
-
 #ifndef STR
 #  define _STR(s) #s
 #  define STR(s)  _STR(s)
@@ -151,6 +130,15 @@
 #define NAMED_OPTS(po) \
         (((po)->fOptSet & (OPTPROC_SHORTOPT | OPTPROC_LONGOPT)) == 0)
 
+typedef int tSuccess;
+#define SUCCESS         ((tSuccess)0)
+#define FAILURE         ((tSuccess)-1)
+#define PROBLEM         ((tSuccess)1)
+#define SUCCEEDED(r)    ((r)==SUCCESS)
+#define SUCCESSFUL(r)   SUCCEEDED(r)
+#define FAILED(r)       ((r)<SUCCESS)
+#define GLITCH(r)       ((r)>SUCCESS)
+
 /*
  *  The pager state is used by doPagedUsage() procedure.
  *  When it runs, it sets itself up to be called again on exit.
@@ -167,6 +155,27 @@ typedef enum {
 
 extern tePagerState pagerState;
 
+typedef enum {
+    ENV_ALL,
+    ENV_IMM,
+    ENV_NON_IMM
+} teEnvPresetType;
+
+typedef enum {
+    TOPT_UNDEFINED = 0,
+    TOPT_SHORT,
+    TOPT_LONG,
+    TOPT_DEFAULT
+} teOptType;
+
+typedef struct {
+    tOptDesc*  pOD;
+    u_long     flags;
+    teOptType  optType;
+    int        argType;
+    char*      pzOptArg;
+} tOptState;
+
 #ifdef MEMDEBUG
    extern void* ag_alloc( size_t, const char* );
    extern void* ag_realloc( void*, size_t, const char* );
@@ -181,5 +190,17 @@ extern tePagerState pagerState;
 #  define AGFREE( p )      free( p )
 #endif
 
+extern void optionUsage PROTO(( tOptions*, int exitCode ));
+
+/*
+ *  optionSave saves the option state into an RC or INI file in
+ *  the *LAST* defined directory in the papzHomeList.
+ */
+void    optionSave PROTO((   tOptions* pOpts ));
+
 #endif /* AUTOGEN_AUTOOPTS_H */
-/* autoopts.h ends here */
+/*
+ * Local Variables:
+ * c-file-style: "stroustrup"
+ * End:
+ * autoopts.h ends here */
