@@ -1,6 +1,6 @@
 
 /*
- *  options.h  $Id: options.h,v 2.10 2000/03/15 16:01:57 bkorb Exp $
+ *  options.h  $Id: options.h,v 2.26 2000/12/11 22:51:31 bkorb Exp $
  *
  *  This file defines all the global structures and special values
  *  used in the automated option processing library.
@@ -65,21 +65,27 @@
  *  for "opt_name" are available.
  */
 
-#define OPTST_INIT         0x0000  /* Initial compiled value            */
-#define OPTST_SET          0x0001  /* Set via the "SET_OPT()" macro     */
-#define OPTST_PRESET       0x0002  /* Set via an RC/INI file            */
-#define OPTST_DEFINED      0x0004  /* Set via a command line option     */
-#define OPTST_SET_MASK     0x000F  /* mask of flags that show set state */
-#define OPTST_EQUIVALENCE  0x0010  /* selected by equiv'ed option       */
-#define OPTST_DISABLED     0x0020  /* option with disable marker        */
-#define OPTST_NO_INIT      0x0100  /* option cannot be preset           */
-#define OPTST_NUMBER_OPT   0x0200  /* option is number option           */
-#define OPTST_NUMERIC      0x0400  /* option has numeric value          */
-#define OPTST_STACKED      0x0800  /* opt uses stackOptArg procedure    */
-#define OPTST_DISABLEOK    0x1000  /* option may be disabled            */
-#define OPTST_INITENABLED  0x2000  /* option defaults to enabled        */
-#define OPTST_DOCUMENT     0x4000  /* opt is for documentation only     */
-#define OPTST_PERSISTENT   0xFF00  /* mask of flags that do not change  */
+#define OPTST_INIT         0x00000  /* Initial compiled value            */
+#define OPTST_SET          0x00001  /* Set via the "SET_OPT()" macro     */
+#define OPTST_PRESET       0x00002  /* Set via an RC/INI file            */
+#define OPTST_DEFINED      0x00004  /* Set via a command line option     */
+#define OPTST_SET_MASK     0x0000F  /* mask of flags that show set state */
+
+#define OPTST_EQUIVALENCE  0x00010  /* selected by equiv'ed option       */
+#define OPTST_DISABLED     0x00020  /* option is in disabled state       */
+
+#define OPTST_NO_INIT      0x00100  /* option cannot be preset           */
+#define OPTST_NUMBER_OPT   0x00200  /* opt value (flag) is any digit     */
+#define OPTST_STACKED      0x00400  /* opt uses stackOptArg procedure    */
+#define OPTST_INITENABLED  0x00800  /* option defaults to enabled        */
+#define OPTST_ENUMERATION  0x01000  /* opt arg is an enum (keyword list) */
+#define OPTST_BOOLEAN      0x02000  /* opt arg is boolean-valued         */
+#define OPTST_NUMERIC      0x04000  /* opt arg has numeric value         */
+#define OPTST_DOCUMENT     0x08000  /* opt is for documentation only     */
+#define OPTST_IMM          0x10000  /* process option on first pass      */
+#define OPTST_DISABLE_IMM  0x20000  /* process disablement on first pass */
+
+#define OPTST_PERSISTENT   0xFFF00  /* mask of flags that do not change  */
 
 #define SELECTED_OPT( pod )  ( (pod)->fOptState & (OPTST_SET | OPTST_DEFINED))
 #define UNUSED_OPT(   pod )  (((pod)->fOptState & OPTST_SET_MASK) == 0)
@@ -100,18 +106,15 @@
 #define OPTPROC_NONE        0x0000
 #define OPTPROC_LONGOPT     0x0001 /* Process long style options      */
 #define OPTPROC_SHORTOPT    0x0002 /* Process short style "flags"     */
-#define OPTPROC_EXERC       0x0004 /* Preload options from exe's dir  */
-#define OPTPROC_ERRSTOP     0x0008 /* Stop on argument errors         */
-#define OPTPROC_DISABLEOK   0x0010 /* Disabling options are allowed   */
-#define OPTPROC_DISABLEDOPT 0x0020 /* Current option is disabled      */
-#define OPTPROC_NO_REQ_OPT  0x0040 /* no options are required         */
-#define OPTPROC_NUM_OPT     0x0080 /* there is a number option        */
-#define OPTPROC_INITDONE    0x0100 /* have initializations been done? */
-#define OPTPROC_NEGATIONS   0x0200 /* any negation options?           */
-#define OPTPROC_ENVIRON     0x0400 /* check environment?              */
-#define OPTPROC_PLUSMARKS   0x0800 /* Use '+' flag markers?           */
-#define OPTPROC_NO_ARGS     0x1000 /* Disallow remaining arguments    */
-#define OPTPROC_ARGS_REQ    0x2000 /* Require arguments after options */
+#define OPTPROC_ERRSTOP     0x0004 /* Stop on argument errors         */
+#define OPTPROC_DISABLEDOPT 0x0008 /* Current option is disabled      */
+#define OPTPROC_NO_REQ_OPT  0x0010 /* no options are required         */
+#define OPTPROC_NUM_OPT     0x0020 /* there is a number option        */
+#define OPTPROC_INITDONE    0x0040 /* have initializations been done? */
+#define OPTPROC_NEGATIONS   0x0080 /* any negation options?           */
+#define OPTPROC_ENVIRON     0x0100 /* check environment?              */
+#define OPTPROC_NO_ARGS     0x0200 /* Disallow remaining arguments    */
+#define OPTPROC_ARGS_REQ    0x0400 /* Require arguments after options */
 
 #define STMTS(s)  do { s; } while (0)
 
@@ -129,7 +132,6 @@
 #define tCUC      const unsigned char
 #define tUC       unsigned char
 #define tCC       const char
-#define tUS       unsigned short
 #define tUI       unsigned int
 #define tUL       unsigned long
 
@@ -201,8 +203,8 @@ struct optDesc {
     tCUC              optMinCt;
     tCUC              optMaxCt;
 
-    tUC               optOccCt;         /* PUBLIC */
-    tUS               fOptState;        /* PUBLIC */
+    tUL               optOccCt;         /* PUBLIC */
+    tUL               fOptState;        /* PUBLIC */
     char*             pzLastArg;        /* PUBLIC */
     void*             optCookie;        /* PUBLIC */
 
@@ -235,7 +237,7 @@ struct specOptIndex {
  *  fields.  This way, the "optionProcess()" routine may exit with an
  *  informative message instead of, for example, page faulting.
  */
-#define  OPTIONS_STRUCT_VERSION  16389
+#define  OPTIONS_STRUCT_VERSION  36992
 
 struct options {
     const int         structVersion;
@@ -250,7 +252,7 @@ struct options {
     const char*       pzUsageTitle;
     const char*       pzExplain;
     const char*       pzDetail;
-    const char*       pzDetailFile;
+    void*             futureUse1;
     tpUsageProc       pUsageProc;
     tUI               fOptSet;
     tUI               curOptIdx;
@@ -259,50 +261,125 @@ struct options {
     const int         optCt;
     const int         presetOptCt;
     tOptDesc*         pOptDesc;
+    int               origArgCt;
+    char**            origArgVect;
+    void*             futureUse2;
 };
 
 #ifdef  __cplusplus
 extern "C" {
 #endif
 
+#if defined( __STDC__ ) || defined( __cplusplus )
+#  ifndef PROTO
+#    define PROTO(s) s
+#  endif
+
+/*
+ *  All this "DEF_PROC_*" weirdness is so that I can compile the option
+ *  processing code with a K&R compiler.  Yuck.  Don't use them unless
+ *  you really have to.  (Still better than ugly if-defs, tho.)
+ */
+#define DEF_PROC_0( id ) \
+          id( void ); \
+          id( void )
+
+#define DEF_PROC_1( id, a1_type, a1_name ) \
+          id( a1_type a1_name ); \
+          id( a1_type a1_name )
+
+#define DEF_PROC_2( id, a1t, a1n, a2t, a2n ) \
+          id( a1t a1n, a2t a2n ); \
+          id( a1t a1n, a2t a2n )
+
+#define DEF_PROC_3( id, a1t, a1n, a2t, a2n, a3t, a3n ) \
+          id( a1t a1n, a2t a2n, a3t a3n ); \
+          id( a1t a1n, a2t a2n, a3t a3n )
+
+#define DEF_PROC_4( id, a1t, a1n, a2t, a2n, a3t, a3n, a4t, a4n ) \
+          id( a1t a1n, a2t a2n, a3t a3n, a4t a4n ); \
+          id( a1t a1n, a2t a2n, a3t a3n, a4t a4n )
+
+#else
+#  ifndef PROTO
+#    define PROTO(s) ()
+#  endif
+
+#define DEF_PROC_0( id ) \
+          id()
+
+#define DEF_PROC_1( id, a1_type, a1_name ) \
+          id( a1_name ) a1_type a1_name;
+
+#define DEF_PROC_2( id, a1t, a1n, a2t, a2n ) \
+          id( a1n, a2n ) a1t a1n; a2t a2n;
+
+#define DEF_PROC_3( id, a1t, a1n, a2t, a2n, a3t, a3n ) \
+          id( a1n, a2n, a3n ) \
+          a1t a1n; a2t a2n; a3t a3n;
+
+#define DEF_PROC_4( id, a1t, a1n, a2t, a2n, a3t, a3n, a4t, a4n ) \
+          id( a1n, a2n, a3n ) \
+          a1t a1n; a2t a2n; a3t a3n; a4t a4n;
+
+#  ifndef void
+#    define void int
+#  endif
+
+#  ifndef const
+#    define const
+#  endif
+#endif
+
 /*
  * Return the full version string compiled into the library.
  */
-const char	*optionVersion( void );
+const char	*optionVersion PROTO(( void ));
 
 /*
  *  optionProcess scans the argument flags completely.
  *  The return value is the argument index of the first argument
  *  following the flag list, per "getopt(3)".
  */
-int     optionProcess( tOptions* pOpts, int argCt, char** argVect );
-
-/*
- *  optionSave saves the option state into an RC or INI file in
- *  the *LAST* defined directory in the papzHomeList.
- */
-void    optionSave(   tOptions* pOpts );
+int     optionProcess PROTO(( tOptions* pOpts, int argCt, char** argVect ));
 
 /*
  *  optionLoadLine will load an option in the text buffer as
  *  a normal (not preset) option.
  */
-void    optionLoadLine( tOptions* pOpts, char*  pzLine );
+void    optionLoadLine PROTO(( tOptions* pOpts, char*  pzLine ));
 
 /*
  *  putBourneShell outputs the option state to standard out in a way
  *  that the Bourne and Korn shells can interpret.
  */
-void    putBourneShell( tOptions* pOpts );
+void    putBourneShell PROTO(( tOptions* pOpts ));
+
+/*
+ *  export_options_to_guile will emit a series of guile commands so
+ *  that the Scheme code can query the state of any of the defined options.
+ */
+void    export_options_to_guile PROTO(( tOptions* ));
+
+/*
+ *  doEnumerationOption will find a string in a table of strings and
+ *  return the index cast as a char* value.  USAGE will be invoked if
+ *  the string is not found in the table.
+ */
+char*   optionEnumerationVal PROTO(( tOptions*, tOptDesc*, tCC**, int ));
 
 /*
  *  stackOptArg saves the option argument into an option-specific list.
  *  It will allocate the list the first time and extend it as needed.
  */
-tOptProc stackOptArg, unstackOptArg;
+tOptProc stackOptArg, unstackOptArg, optionBooleanVal, optionNumericVal;
 
 #ifdef  __cplusplus
 }
 #endif
 #endif /* AUTOGEN_OPTIONS_H */
-/* options.h ends here */
+/*
+ * Local Variables:
+ * c-file-style: "stroustrup"
+ * End:
+ * options.h ends here */
