@@ -3,12 +3,12 @@
 /* --- fake the preprocessor into handlng portability */
 
 /*
- * Author:           Gary V Vaughan <gvv@techie.com>
+ * Author:           Gary V Vaughan <gvaughan@oranda.demon.co.uk>
  * Created:          Mon Jun 30 15:54:46 1997
  * Last Modified:     Fri Jul 30 12:34:46 1999
- *            by:     Gary V. Vaughan <gvv@techie.com>
+ *            by:     Gary V. Vaughan <gary@oranda.demon.co.uk>
  *
- * $Id: compat.h,v 2.5 1999/07/30 14:48:46 bkorb Exp $
+ * $Id: compat.h,v 2.6 2000/03/05 18:41:59 bruce Exp $
  */
 #ifndef COMPAT_H
 #define COMPAT_H 1
@@ -27,6 +27,17 @@
 #  define STATIC static
 #endif
 
+#ifndef PARAMS
+#  if __STDC__
+#    ifndef NOPROTOS
+#      define PARAMS(args)      args
+#    endif
+#  endif
+#  ifndef PARAMS
+#    define PARAMS(args)        ()
+#  endif
+#endif
+
 #include <config.h>
 
 #ifdef HAVE_SYS_TYPES_H
@@ -163,7 +174,7 @@
 #endif
 
 #ifndef HAVE_PATHFIND
-  EXTERN char *pathfind (/* const char *, const char *, const char * */);
+  EXTERN char *pathfind PARAMS((const char *, const char *, const char *));
 #endif
 
 # if defined (HAVE_DIRENT_H)
