@@ -1,13 +1,13 @@
 /*  -*- Mode: C -*-
  *
- *  $Id: getdefs.c,v 2.26 2001/03/04 20:15:05 bkorb Exp $
+ *  $Id: getdefs.c,v 2.27 2001/04/22 18:06:15 bkorb Exp $
  *
  *    getdefs copyright 1999-2001 Bruce Korb
  *
  *  Author:            Bruce Korb <bkorb@gnu.org>
  *  Maintainer:        Bruce Korb <bkorb@gnu.org>
  *  Created:           Mon Jun 30 15:35:12 1997
- *  Last Modified:     $Date: 2001/03/04 20:15:05 $
+ *  Last Modified:     $Date: 2001/04/22 18:06:15 $
  *            by:      Bruce Korb <bkorb@gnu.org>
  */
 
@@ -1052,7 +1052,7 @@ buildPreamble(
         return FAILURE;
     }
 
-    *pzDef++ = NUL;
+    *pzDef = NUL;
 
     /*
      *  Now start the output.  First, the "#line" directive,
@@ -1099,6 +1099,7 @@ buildPreamble(
 
     *ppzOut = pzOut;
     *ppzDef = pzDef;
+    *pzDef  = '\n';  /* restore the newline.  Used in pattern match */
 
     /*
      *  Returning "PROBLEM" means the caller must emit the "#endif\n"
